@@ -1,7 +1,7 @@
 //! Alacritty - The GPU Enhanced Terminal.
 
 #![warn(rust_2018_idioms, future_incompatible)]
-#![deny(clippy::all, clippy::if_not_else, clippy::enum_glob_use, clippy::wrong_pub_self_convention)]
+#![deny(clippy::all, clippy::if_not_else, clippy::enum_glob_use)]
 #![cfg_attr(feature = "cargo-clippy", deny(warnings))]
 // With the default subsystem, 'console', windows creates an additional console
 // window for the program.
