@@ -5,106 +5,85 @@
 //! when text is added/removed/scrolled on the screen. The selection should
 //! also be cleared if the user clicks off of the selection.
 
-use std::convert::TryFrom;
+use std::cmp::min;
 use std::mem;
 use std::ops::{Bound, Range, RangeBounds};
 
 use crate::ansi::CursorShape;
-use crate::grid::{Dimensions, Grid, GridCell};
-use crate::index::{Column, Line, Point, Side};
-use crate::term::cell::Flags;
+use crate::grid::{Dimensions, GridCell, Indexed};
+use crate::index::{Boundary, Column, Line, Point, Side};
+use crate::term::cell::{Cell, Flags};
 use crate::term::Term;
 
 /// A Point and side within that point.
 #[derive(Debug, Copy, Clone, PartialEq)]
 struct Anchor {
-    point: Point<usize>,
+    point: Point,
     side: Side,
 }
 
 impl Anchor {
-    fn new(point: Point<usize>, side: Side) -> Anchor {
+    fn new(point: Point, side: Side) -> Anchor {
         Anchor { point, side }
     }
 }
 
 /// Represents a range of selected cells.
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
-pub struct SelectionRange<L = usize> {
+pub struct SelectionRange {
     /// Start point, top left of the selection.
-    pub start: Point<L>,
+    pub start: Point,
     /// End point, bottom right of the selection.
-    pub end: Point<L>,
+    pub end: Point,
     /// Whether this selection is a block selection.
     pub is_block: bool,
 }
 
-impl<L> SelectionRange<L> {
-    pub fn new(start: Point<L>, end: Point<L>, is_block: bool) -> Self {
+impl SelectionRange {
+    pub fn new(start: Point, end: Point, is_block: bool) -> Self {
         Self { start, end, is_block }
     }
-
-    /// Check if a point lies within the selection.
-    pub fn contains(&self, point: Point<L>) -> bool
-    where
-        L: PartialEq + PartialOrd,
-    {
-        self.start.line <= point.line
-            && self.end.line >= point.line
-            && (self.start.col <= point.col || (self.start.line != point.line && !self.is_block))
-            && (self.end.col >= point.col || (self.end.line != point.line && !self.is_block))
-    }
 }
 
-impl SelectionRange<Line> {
+impl SelectionRange {
+    /// Check if a point lies within the selection.
+    pub fn contains(&self, point: Point) -> bool {
+        self.start.line <= point.line
+            && self.end.line >= point.line
+            && (self.start.column <= point.column
+                || (self.start.line != point.line && !self.is_block))
+            && (self.end.column >= point.column || (self.end.line != point.line && !self.is_block))
+    }
+
     /// Check if the cell at a point is part of the selection.
-    pub fn contains_cell<T>(
+    pub fn contains_cell(
         &self,
-        grid: &Grid<T>,
+        indexed: &Indexed<&Cell>,
         point: Point,
-        cursor_point: Point,
-        cursor_shape: CursorShape,
-    ) -> bool
-    where
-        T: GridCell,
-    {
+        shape: CursorShape,
+    ) -> bool {
         // Do not invert block cursor at selection boundaries.
-        if cursor_shape == CursorShape::Block
-            && cursor_point == point
-            && (self.start == point
-                || self.end == point
+        if shape == CursorShape::Block
+            && point == indexed.point
+            && (self.start == indexed.point
+                || self.end == indexed.point
                 || (self.is_block
-                    && ((self.start.line == point.line && self.end.col == point.col)
-                        || (self.end.line == point.line && self.start.col == point.col))))
+                    && ((self.start.line == indexed.point.line
+                        && self.end.column == indexed.point.column)
+                        || (self.end.line == indexed.point.line
+                            && self.start.column == indexed.point.column))))
         {
             return false;
         }
 
         // Point itself is selected.
-        if self.contains(point) {
+        if self.contains(indexed.point) {
             return true;
         }
 
-        let num_cols = grid.cols();
-
-        // Convert to absolute coordinates to adjust for the display offset.
-        let buffer_point = grid.visible_to_buffer(point);
-        let cell = &grid[buffer_point];
-
-        // Check if wide char's spacers are selected.
-        if cell.flags().contains(Flags::WIDE_CHAR) {
-            let prev = point.sub(num_cols, 1);
-            let buffer_prev = grid.visible_to_buffer(prev);
-            let next = point.add(num_cols, 1);
-
-            // Check trailing spacer.
-            self.contains(next)
-                // Check line-wrapping, leading spacer.
-                || (grid[buffer_prev].flags().contains(Flags::LEADING_WIDE_CHAR_SPACER)
-                    && self.contains(prev))
-        } else {
-            false
-        }
+        // Check if a wide char's trailing spacer is selected.
+        indexed.cell.flags().contains(Flags::WIDE_CHAR)
+            && self.contains(Point::new(indexed.point.line, indexed.point.column + 1))
     }
 }
 
@@ -142,7 +121,7 @@ pub struct Selection {
 }
 
 impl Selection {
-    pub fn new(ty: SelectionType, location: Point<usize>, side: Side) -> Selection {
+    pub fn new(ty: SelectionType, location: Point, side: Side) -> Selection {
         Self {
             region: Range { start: Anchor::new(location, side), end: Anchor::new(location, side) },
             ty,
@@ -150,7 +129,7 @@ impl Selection {
     }
 
     /// Update the end of the selection.
-    pub fn update(&mut self, point: Point<usize>, side: Side) {
+    pub fn update(&mut self, point: Point, side: Side) {
         self.region.end = Anchor::new(point, side);
     }
 
@@ -158,56 +137,52 @@ impl Selection {
         mut self,
         dimensions: &D,
         range: &Range<Line>,
-        delta: isize,
+        delta: i32,
     ) -> Option<Selection> {
-        let num_lines = dimensions.screen_lines().0;
-        let num_cols = dimensions.cols().0;
-        let range_bottom = range.start.0;
-        let range_top = range.end.0;
+        let bottommost_line = dimensions.bottommost_line();
+        let range_bottom = range.end;
+        let range_top = range.start;
 
         let (mut start, mut end) = (&mut self.region.start, &mut self.region.end);
-        if Selection::points_need_swap(start.point, end.point) {
+        if start.point > end.point {
             mem::swap(&mut start, &mut end);
         }
 
         // Rotate start of selection.
-        if (start.point.line < range_top || range_top == num_lines)
-            && start.point.line >= range_bottom
-        {
-            start.point.line = usize::try_from(start.point.line as isize + delta).unwrap_or(0);
+        if (start.point.line >= range_top || range_top == 0) && start.point.line < range_bottom {
+            start.point.line = min(start.point.line - delta, bottommost_line);
 
             // If end is within the same region, delete selection once start rotates out.
-            if start.point.line < range_bottom && end.point.line >= range_bottom {
+            if start.point.line >= range_bottom && end.point.line < range_bottom {
                 return None;
             }
 
             // Clamp selection to start of region.
-            if start.point.line >= range_top && range_top != num_lines {
+            if start.point.line < range_top && range_top != 0 {
                 if self.ty != SelectionType::Block {
-                    start.point.col = Column(0);
+                    start.point.column = Column(0);
                     start.side = Side::Left;
                 }
-                start.point.line = range_top - 1;
+                start.point.line = range_top;
             }
         }
 
         // Rotate end of selection.
-        if (end.point.line < range_top || range_top == num_lines) && end.point.line >= range_bottom
-        {
-            end.point.line = usize::try_from(end.point.line as isize + delta).unwrap_or(0);
+        if (end.point.line >= range_top || range_top == 0) && end.point.line < range_bottom {
+            end.point.line = min(end.point.line - delta, bottommost_line);
 
             // Delete selection if end has overtaken the start.
-            if end.point.line > start.point.line {
+            if end.point.line < start.point.line {
                 return None;
             }
 
             // Clamp selection to end of region.
-            if end.point.line < range_bottom {
+            if end.point.line >= range_bottom {
                 if self.ty != SelectionType::Block {
-                    end.point.col = Column(num_cols - 1);
+                    end.point.column = dimensions.last_column();
                     end.side = Side::Right;
                 }
-                end.point.line = range_bottom;
+                end.point.line = range_bottom - 1;
             }
         }
 
@@ -218,7 +193,7 @@ impl Selection {
         match self.ty {
             SelectionType::Simple => {
                 let (mut start, mut end) = (self.region.start, self.region.end);
-                if Self::points_need_swap(start.point, end.point) {
+                if start.point > end.point {
                     mem::swap(&mut start, &mut end);
                 }
 
@@ -228,7 +203,7 @@ impl Selection {
                     || (start.side == Side::Right
                         && end.side == Side::Left
                         && (start.point.line == end.point.line)
-                        && start.point.col + 1 == end.point.col)
+                        && start.point.column + 1 == end.point.column)
             },
             SelectionType::Block => {
                 let (start, end) = (self.region.start, self.region.end);
@@ -236,11 +211,11 @@ impl Selection {
                 // Block selection is empty when the points' columns and sides are identical
                 // or two cells with adjacent columns have the sides right -> left,
                 // regardless of their lines
-                (start.point.col == end.point.col && start.side == end.side)
-                    || (start.point.col + 1 == end.point.col
+                (start.point.column == end.point.column && start.side == end.side)
+                    || (start.point.column + 1 == end.point.column
                         && start.side == Side::Right
                         && end.side == Side::Left)
-                    || (end.point.col + 1 == start.point.col
+                    || (end.point.column + 1 == start.point.column
                         && start.side == Side::Left
                         && end.side == Side::Right)
             },
@@ -249,27 +224,27 @@ impl Selection {
     }
 
     /// Check whether selection contains any point in a given range.
-    pub fn intersects_range<R: RangeBounds<usize>>(&self, range: R) -> bool {
+    pub fn intersects_range<R: RangeBounds<Line>>(&self, range: R) -> bool {
         let mut start = self.region.start.point.line;
         let mut end = self.region.end.point.line;
 
-        if Self::points_need_swap(self.region.start.point, self.region.end.point) {
+        if start > end {
             mem::swap(&mut start, &mut end);
         }
 
-        let range_start = match range.start_bound() {
+        let range_top = match range.start_bound() {
             Bound::Included(&range_start) => range_start,
-            Bound::Excluded(&range_start) => range_start.saturating_add(1),
-            Bound::Unbounded => 0,
+            Bound::Excluded(&range_start) => range_start + 1,
+            Bound::Unbounded => Line(i32::min_value()),
         };
 
-        let range_end = match range.end_bound() {
+        let range_bottom = match range.end_bound() {
             Bound::Included(&range_end) => range_end,
-            Bound::Excluded(&range_end) => range_end.saturating_sub(1),
-            Bound::Unbounded => usize::max_value(),
+            Bound::Excluded(&range_end) => range_end - 1,
+            Bound::Unbounded => Line(i32::max_value()),
         };
 
-        range_start <= start && range_end >= end
+        range_bottom >= start && range_top <= end
     }
 
     /// Expand selection sides to include all cells.
@@ -277,12 +252,13 @@ impl Selection {
         let (start, end) = (self.region.start.point, self.region.end.point);
         let (start_side, end_side) = match self.ty {
             SelectionType::Block
-                if start.col > end.col || (start.col == end.col && start.line < end.line) =>
+                if start.column > end.column
+                    || (start.column == end.column && start.line > end.line) =>
             {
                 (Side::Right, Side::Left)
             },
             SelectionType::Block => (Side::Left, Side::Right),
-            _ if Self::points_need_swap(start, end) => (Side::Right, Side::Left),
+            _ if start > end => (Side::Right, Side::Left),
             _ => (Side::Left, Side::Right),
         };
 
@@ -293,66 +269,34 @@ impl Selection {
     /// Convert selection to grid coordinates.
     pub fn to_range<T>(&self, term: &Term<T>) -> Option<SelectionRange> {
         let grid = term.grid();
-        let num_cols = grid.cols();
+        let columns = grid.columns();
 
         // Order start above the end.
         let mut start = self.region.start;
         let mut end = self.region.end;
 
-        if Self::points_need_swap(start.point, end.point) {
+        if start.point > end.point {
             mem::swap(&mut start, &mut end);
         }
 
-        // Clamp to inside the grid buffer.
-        let is_block = self.ty == SelectionType::Block;
-        let (start, end) = Self::grid_clamp(start, end, is_block, grid.total_lines()).ok()?;
+        // Clamp selection to within grid boundaries.
+        if end.point.line < term.topmost_line() {
+            return None;
+        }
+        start.point = start.point.grid_clamp(term, Boundary::Grid);
 
         match self.ty {
-            SelectionType::Simple => self.range_simple(start, end, num_cols),
+            SelectionType::Simple => self.range_simple(start, end, columns),
             SelectionType::Block => self.range_block(start, end),
             SelectionType::Semantic => Some(Self::range_semantic(term, start.point, end.point)),
             SelectionType::Lines => Some(Self::range_lines(term, start.point, end.point)),
         }
     }
 
-    /// Bring start and end points in the correct order.
-    fn points_need_swap(start: Point<usize>, end: Point<usize>) -> bool {
-        start.line < end.line || start.line == end.line && start.col > end.col
-    }
-
-    /// Clamp selection inside grid to prevent OOB.
-    fn grid_clamp(
-        mut start: Anchor,
-        end: Anchor,
-        is_block: bool,
-        lines: usize,
-    ) -> Result<(Anchor, Anchor), ()> {
-        // Clamp selection inside of grid to prevent OOB.
-        if start.point.line >= lines {
-            // Remove selection if it is fully out of the grid.
-            if end.point.line >= lines {
-                return Err(());
-            }
-
-            // Clamp to grid if it is still partially visible.
-            if !is_block {
-                start.side = Side::Left;
-                start.point.col = Column(0);
-            }
-            start.point.line = lines - 1;
-        }
-
-        Ok((start, end))
-    }
-
-    fn range_semantic<T>(
-        term: &Term<T>,
-        mut start: Point<usize>,
-        mut end: Point<usize>,
-    ) -> SelectionRange {
+    fn range_semantic<T>(term: &Term<T>, mut start: Point, mut end: Point) -> SelectionRange {
         if start == end {
             if let Some(matching) = term.bracket_search(start) {
-                if (matching.line == start.line && matching.col < start.col)
+                if (matching.line == start.line && matching.column < start.column)
                     || (matching.line > start.line)
                 {
                     start = matching;
@@ -364,19 +308,15 @@ impl Selection {
             }
         }
 
-        start = term.semantic_search_left(start);
-        end = term.semantic_search_right(end);
+        let start = term.semantic_search_left(start);
+        let end = term.semantic_search_right(end);
 
         SelectionRange { start, end, is_block: false }
     }
 
-    fn range_lines<T>(
-        term: &Term<T>,
-        mut start: Point<usize>,
-        mut end: Point<usize>,
-    ) -> SelectionRange {
-        start = term.line_search_left(start);
-        end = term.line_search_right(end);
+    fn range_lines<T>(term: &Term<T>, start: Point, end: Point) -> SelectionRange {
+        let start = term.line_search_left(start);
+        let end = term.line_search_right(end);
 
         SelectionRange { start, end, is_block: false }
     }
@@ -385,7 +325,7 @@ impl Selection {
         &self,
         mut start: Anchor,
         mut end: Anchor,
-        num_cols: Column,
+        columns: usize,
     ) -> Option<SelectionRange> {
         if self.is_empty() {
             return None;
@@ -394,21 +334,22 @@ impl Selection {
         // Remove last cell if selection ends to the left of a cell.
         if end.side == Side::Left && start.point != end.point {
             // Special case when selection ends to left of first cell.
-            if end.point.col == Column(0) {
-                end.point.col = num_cols - 1;
-                end.point.line += 1;
+            if end.point.column == 0 {
+                end.point.column = Column(columns - 1);
+                end.point.line -= 1;
             } else {
-                end.point.col -= 1;
+                end.point.column -= 1;
             }
         }
 
         // Remove first cell if selection starts at the right of a cell.
         if start.side == Side::Right && start.point != end.point {
-            start.point.col += 1;
+            start.point.column += 1;
 
             // Wrap to next line when selection starts to the right of last column.
-            if start.point.col == num_cols {
-                start.point = Point::new(start.point.line.saturating_sub(1), Column(0));
+            if start.point.column == columns {
+                start.point.column = Column(0);
+                start.point.line += 1;
             }
         }
 
@@ -421,19 +362,19 @@ impl Selection {
         }
 
         // Always go top-left -> bottom-right.
-        if start.point.col > end.point.col {
+        if start.point.column > end.point.column {
             mem::swap(&mut start.side, &mut end.side);
-            mem::swap(&mut start.point.col, &mut end.point.col);
+            mem::swap(&mut start.point.column, &mut end.point.column);
         }
 
         // Remove last cell if selection ends to the left of a cell.
-        if end.side == Side::Left && start.point != end.point && end.point.col.0 > 0 {
-            end.point.col -= 1;
+        if end.side == Side::Left && start.point != end.point && end.point.column.0 > 0 {
+            end.point.column -= 1;
         }
 
         // Remove first cell if selection starts at the right of a cell.
         if start.side == Side::Right && start.point != end.point {
-            start.point.col += 1;
+            start.point.column += 1;
         }
 
         Some(SelectionRange { start: start.point, end: end.point, is_block: true })
@@ -454,18 +395,12 @@ mod tests {
     use super::*;
 
     use crate::config::MockConfig;
-    use crate::event::{Event, EventListener};
-    use crate::index::{Column, Line, Point, Side};
+    use crate::index::{Column, Point, Side};
     use crate::term::{SizeInfo, Term};
 
-    struct Mock;
-    impl EventListener for Mock {
-        fn send_event(&self, _event: Event) {}
-    }
-
-    fn term(height: usize, width: usize) -> Term<Mock> {
+    fn term(height: usize, width: usize) -> Term<()> {
         let size = SizeInfo::new(width as f32, height as f32, 1.0, 1.0, 0.0, 0.0, false);
-        Term::new(&MockConfig::default(), size, Mock)
+        Term::new(&MockConfig::default(), size, ())
     }
 
     /// Test case of single cell selection.
@@ -475,7 +410,7 @@ mod tests {
     /// 3. [BE]
     #[test]
     fn single_cell_left_to_right() {
-        let location = Point { line: 0, col: Column(0) };
+        let location = Point::new(Line(0), Column(0));
         let mut selection = Selection::new(SelectionType::Simple, location, Side::Left);
         selection.update(location, Side::Right);
 
@@ -493,7 +428,7 @@ mod tests {
     /// 3. [EB]
     #[test]
     fn single_cell_right_to_left() {
-        let location = Point { line: 0, col: Column(0) };
+        let location = Point::new(Line(0), Column(0));
         let mut selection = Selection::new(SelectionType::Simple, location, Side::Right);
         selection.update(location, Side::Left);
 
@@ -512,8 +447,8 @@ mod tests {
     #[test]
     fn between_adjacent_cells_left_to_right() {
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(0, Column(0)), Side::Right);
-        selection.update(Point::new(0, Column(1)), Side::Left);
+            Selection::new(SelectionType::Simple, Point::new(Line(0), Column(0)), Side::Right);
+        selection.update(Point::new(Line(0), Column(1)), Side::Left);
 
         assert_eq!(selection.to_range(&term(1, 2)), None);
     }
@@ -526,8 +461,8 @@ mod tests {
     #[test]
     fn between_adjacent_cells_right_to_left() {
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(0, Column(1)), Side::Left);
-        selection.update(Point::new(0, Column(0)), Side::Right);
+            Selection::new(SelectionType::Simple, Point::new(Line(0), Column(1)), Side::Left);
+        selection.update(Point::new(Line(0), Column(0)), Side::Right);
 
         assert_eq!(selection.to_range(&term(1, 2)), None);
     }
@@ -543,12 +478,12 @@ mod tests {
     #[test]
     fn across_adjacent_lines_upward_final_cell_exclusive() {
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(1, Column(1)), Side::Right);
-        selection.update(Point::new(0, Column(1)), Side::Right);
+            Selection::new(SelectionType::Simple, Point::new(Line(0), Column(1)), Side::Right);
+        selection.update(Point::new(Line(1), Column(1)), Side::Right);
 
         assert_eq!(selection.to_range(&term(2, 5)).unwrap(), SelectionRange {
-            start: Point::new(1, Column(2)),
-            end: Point::new(0, Column(1)),
+            start: Point::new(Line(0), Column(2)),
+            end: Point::new(Line(1), Column(1)),
             is_block: false,
         });
     }
@@ -566,73 +501,73 @@ mod tests {
     #[test]
     fn selection_bigger_then_smaller() {
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(0, Column(1)), Side::Right);
-        selection.update(Point::new(1, Column(1)), Side::Right);
-        selection.update(Point::new(1, Column(0)), Side::Right);
+            Selection::new(SelectionType::Simple, Point::new(Line(1), Column(1)), Side::Right);
+        selection.update(Point::new(Line(0), Column(1)), Side::Right);
+        selection.update(Point::new(Line(0), Column(0)), Side::Right);
 
         assert_eq!(selection.to_range(&term(2, 5)).unwrap(), SelectionRange {
-            start: Point::new(1, Column(1)),
-            end: Point::new(0, Column(1)),
+            start: Point::new(Line(0), Column(1)),
+            end: Point::new(Line(1), Column(1)),
             is_block: false,
         });
     }
 
     #[test]
     fn line_selection() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Lines, Point::new(0, Column(1)), Side::Left);
-        selection.update(Point::new(5, Column(1)), Side::Right);
-        selection = selection.rotate(&size, &(Line(0)..size.0), 7).unwrap();
+            Selection::new(SelectionType::Lines, Point::new(Line(9), Column(1)), Side::Left);
+        selection.update(Point::new(Line(4), Column(1)), Side::Right);
+        selection = selection.rotate(&size, &(Line(0)..Line(size.0 as i32)), 4).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(9, Column(0)),
-            end: Point::new(7, Column(4)),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(0), Column(0)),
+            end: Point::new(Line(5), Column(4)),
             is_block: false,
         });
     }
 
     #[test]
     fn semantic_selection() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Semantic, Point::new(0, Column(3)), Side::Left);
-        selection.update(Point::new(5, Column(1)), Side::Right);
-        selection = selection.rotate(&size, &(Line(0)..size.0), 7).unwrap();
+            Selection::new(SelectionType::Semantic, Point::new(Line(9), Column(3)), Side::Left);
+        selection.update(Point::new(Line(4), Column(1)), Side::Right);
+        selection = selection.rotate(&size, &(Line(0)..Line(size.0 as i32)), 4).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(9, Column(0)),
-            end: Point::new(7, Column(3)),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(0), Column(1)),
+            end: Point::new(Line(5), Column(3)),
             is_block: false,
         });
     }
 
     #[test]
     fn simple_selection() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(0, Column(3)), Side::Right);
-        selection.update(Point::new(5, Column(1)), Side::Right);
-        selection = selection.rotate(&size, &(Line(0)..size.0), 7).unwrap();
+            Selection::new(SelectionType::Simple, Point::new(Line(9), Column(3)), Side::Right);
+        selection.update(Point::new(Line(4), Column(1)), Side::Right);
+        selection = selection.rotate(&size, &(Line(0)..Line(size.0 as i32)), 4).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(9, Column(0)),
-            end: Point::new(7, Column(3)),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(0), Column(2)),
+            end: Point::new(Line(5), Column(3)),
             is_block: false,
         });
     }
 
     #[test]
     fn block_selection() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Block, Point::new(0, Column(3)), Side::Right);
-        selection.update(Point::new(5, Column(1)), Side::Right);
-        selection = selection.rotate(&size, &(Line(0)..size.0), 7).unwrap();
+            Selection::new(SelectionType::Block, Point::new(Line(9), Column(3)), Side::Right);
+        selection.update(Point::new(Line(4), Column(1)), Side::Right);
+        selection = selection.rotate(&size, &(Line(0)..Line(size.0 as i32)), 4).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(9, Column(2)),
-            end: Point::new(7, Column(3)),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(0), Column(2)),
+            end: Point::new(Line(5), Column(3)),
             is_block: true
         });
     }
@@ -640,72 +575,72 @@ mod tests {
     #[test]
     fn simple_is_empty() {
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(0, Column(0)), Side::Right);
+            Selection::new(SelectionType::Simple, Point::new(Line(1), Column(0)), Side::Right);
         assert!(selection.is_empty());
-        selection.update(Point::new(0, Column(1)), Side::Left);
+        selection.update(Point::new(Line(1), Column(1)), Side::Left);
         assert!(selection.is_empty());
-        selection.update(Point::new(1, Column(0)), Side::Right);
+        selection.update(Point::new(Line(0), Column(0)), Side::Right);
         assert!(!selection.is_empty());
     }
 
     #[test]
     fn block_is_empty() {
         let mut selection =
-            Selection::new(SelectionType::Block, Point::new(0, Column(0)), Side::Right);
+            Selection::new(SelectionType::Block, Point::new(Line(1), Column(0)), Side::Right);
         assert!(selection.is_empty());
-        selection.update(Point::new(0, Column(1)), Side::Left);
+        selection.update(Point::new(Line(1), Column(1)), Side::Left);
         assert!(selection.is_empty());
-        selection.update(Point::new(0, Column(1)), Side::Right);
+        selection.update(Point::new(Line(1), Column(1)), Side::Right);
         assert!(!selection.is_empty());
-        selection.update(Point::new(1, Column(0)), Side::Right);
+        selection.update(Point::new(Line(0), Column(0)), Side::Right);
         assert!(selection.is_empty());
-        selection.update(Point::new(1, Column(1)), Side::Left);
+        selection.update(Point::new(Line(0), Column(1)), Side::Left);
         assert!(selection.is_empty());
-        selection.update(Point::new(1, Column(1)), Side::Right);
+        selection.update(Point::new(Line(0), Column(1)), Side::Right);
         assert!(!selection.is_empty());
     }
 
     #[test]
     fn rotate_in_region_up() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(2, Column(3)), Side::Right);
-        selection.update(Point::new(5, Column(1)), Side::Right);
-        selection = selection.rotate(&size, &(Line(1)..(size.0 - 1)), 4).unwrap();
+            Selection::new(SelectionType::Simple, Point::new(Line(7), Column(3)), Side::Right);
+        selection.update(Point::new(Line(4), Column(1)), Side::Right);
+        selection = selection.rotate(&size, &(Line(1)..Line(size.0 as i32 - 1)), 4).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(8, Column(0)),
-            end: Point::new(6, Column(3)),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(1), Column(0)),
+            end: Point::new(Line(3), Column(3)),
             is_block: false,
         });
     }
 
     #[test]
     fn rotate_in_region_down() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Simple, Point::new(5, Column(3)), Side::Right);
-        selection.update(Point::new(8, Column(1)), Side::Left);
-        selection = selection.rotate(&size, &(Line(1)..(size.0 - 1)), -5).unwrap();
+            Selection::new(SelectionType::Simple, Point::new(Line(4), Column(3)), Side::Right);
+        selection.update(Point::new(Line(1), Column(1)), Side::Left);
+        selection = selection.rotate(&size, &(Line(1)..Line(size.0 as i32 - 1)), -5).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(3, Column(1)),
-            end: Point::new(1, size.1 - 1),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(6), Column(1)),
+            end: Point::new(Line(8), size.last_column()),
             is_block: false,
         });
     }
 
     #[test]
     fn rotate_in_region_up_block() {
-        let size = (Line(10), Column(5));
+        let size = (10, 5);
         let mut selection =
-            Selection::new(SelectionType::Block, Point::new(2, Column(3)), Side::Right);
-        selection.update(Point::new(5, Column(1)), Side::Right);
-        selection = selection.rotate(&size, &(Line(1)..(size.0 - 1)), 4).unwrap();
+            Selection::new(SelectionType::Block, Point::new(Line(7), Column(3)), Side::Right);
+        selection.update(Point::new(Line(4), Column(1)), Side::Right);
+        selection = selection.rotate(&size, &(Line(1)..Line(size.0 as i32 - 1)), 4).unwrap();
 
-        assert_eq!(selection.to_range(&term(*size.0, *size.1)).unwrap(), SelectionRange {
-            start: Point::new(8, Column(2)),
-            end: Point::new(6, Column(3)),
+        assert_eq!(selection.to_range(&term(size.0, size.1)).unwrap(), SelectionRange {
+            start: Point::new(Line(1), Column(2)),
+            end: Point::new(Line(3), Column(3)),
             is_block: true,
         });
     }
@@ -713,17 +648,17 @@ mod tests {
     #[test]
     fn range_intersection() {
         let mut selection =
-            Selection::new(SelectionType::Lines, Point::new(6, Column(1)), Side::Left);
-        selection.update(Point::new(3, Column(1)), Side::Right);
+            Selection::new(SelectionType::Lines, Point::new(Line(3), Column(1)), Side::Left);
+        selection.update(Point::new(Line(6), Column(1)), Side::Right);
 
         assert!(selection.intersects_range(..));
-        assert!(selection.intersects_range(2..));
-        assert!(selection.intersects_range(2..=4));
-        assert!(selection.intersects_range(2..=7));
-        assert!(selection.intersects_range(4..=5));
-        assert!(selection.intersects_range(5..8));
+        assert!(selection.intersects_range(Line(2)..));
+        assert!(selection.intersects_range(Line(2)..=Line(4)));
+        assert!(selection.intersects_range(Line(2)..=Line(7)));
+        assert!(selection.intersects_range(Line(4)..=Line(5)));
+        assert!(selection.intersects_range(Line(5)..Line(8)));
 
-        assert!(!selection.intersects_range(..=2));
-        assert!(!selection.intersects_range(7..=8));
+        assert!(!selection.intersects_range(..=Line(2)));
+        assert!(!selection.intersects_range(Line(7)..=Line(8)));
     }
 }
